/*
This is a sample code for 
	T. Ooura, 
	"Numerical inversion of the Laplace transform using a 
	continuous Euler transformation"
	RIMS Kokyuroku, Kyoto Univ., 1145, (2000), 188-193.
*/
#include <math.h>
#include <stdio.h>
#ifndef M_PI
#define M_PI        3.14159265358979323846
#endif


double cet_wish(double beta, double x); /* in "cet_wish.c" */
void rdft(int, int, double *); /* in "fftsg_h.c" */


void cet_ilt(int n, double h_pi, double sigma, double eps, 
			 void (*G)(double sr, double si, double *yr, double *yi), double g[])
{
	int j;
	double leps, beta, gamma, yr, yi, w;

	leps = -log(eps);
	beta = M_PI / 2 * n;
	if (beta > 1.5 * leps) beta = 1.5 * leps;
	gamma = 0.5 * leps * h_pi + sigma;
	G(gamma, 0, &yr, &yi);
	w = h_pi * cet_wish(beta, beta);
	g[0] = yr * (h_pi - w);
	G(gamma, M_PI * h_pi * n, &yr, &yi);
	g[1] = yr * w;
	if (n % 2 == 0 && n >= 2) {
		G(gamma, M_PI * h_pi * (n / 2), &yr, &yi);
		w = h_pi * 0.5;
		g[n] = yr * w;
		g[n + 1] = -yi * w;
	}
	for (j = 1; j < n / 2; j++) {
		G(gamma, M_PI * h_pi * j, &yr, &yi);
		w = h_pi * cet_wish(beta, beta - (2 * beta / n) * j);
		g[2 * j] = yr * (h_pi - w);
		g[2 * j + 1] = -yi * (h_pi - w);
		G(gamma, M_PI * h_pi * (n - j), &yr, &yi);
		g[2 * (n - j)] = yr * w;
		g[2 * (n - j) + 1] = -yi * w;
	}
	rdft(2 * n, -1, g);
	for (j = 0; j < 2 * n; j++) {
		g[j] *= exp(gamma / (n * h_pi) * j);
	}
}


void G_1(double sr, double si, double *yr, double *yi) /* y = 1 / (1 + s * s) */
{
	double ur = 1 + sr * sr - si * si;
	double ui = 2 * sr * si;
	double up = 1 / (ur * ur + ui * ui);
	*yr = ur * up;
	*yi = -ui * up;
}


double g_1(double t)
{
	return sin(t); /* Exact function of ILT(G_1) */
}


int main()
{
	double g[65536];
	int n, j;
	double t_max, h_pi, eps, sigma, t;

	n = 1024; /* n = N/2 */
	t_max = 24;
	h_pi = 1.5 / t_max; /* h_pi = h/pi */
	eps = 1.0e-10;
	sigma = 0;

	cet_ilt(n, h_pi, sigma, eps, G_1, g);

	for (j = 0; j < 1.5 * n; j++) {
		t = (1 / (n * h_pi)) * j;
		printf("t= %g,\tg(t)= %g,\terror= %g\n", t, g[j], 
			g[j] - g_1(t));
	}
	printf("N_func_eval= %d\n", n);
	return 0;
}

